@@ -4,7 +4,7 @@ import haiku as hk
 import jax
 import jax.numpy as jnp
 
-from e3nn_jax import Irreps, IrrepsData
+from e3nn_jax import Irreps, IrrepsArray, config
 from e3nn_jax.util import prod
 
 
@@ -26,10 +26,10 @@ def _batch_norm(
     def _roll_avg(curr, update):
         return (1 - momentum) * curr + momentum * jax.lax.stop_gradient(update)
 
-    batch, *size = input.shape
+    batch, *size = input.shape[:-1]
     # TODO add test case for when prod(size) == 0
 
-    input = input.reshape((batch, prod(size)))
+    input = input.reshape((batch, prod(size), -1))
 
     new_means = []
     new_vars = []
@@ -112,8 +112,8 @@ def _batch_norm(
             fields.append(field)  # [batch, sample, mul, repr]
         i_wei += mul
 
-    output = IrrepsData.from_list(input.irreps, fields, (batch, prod(size)))
-    output = output.reshape((batch,) + tuple(size))
+    output = IrrepsArray.from_list(input.irreps, fields, (batch, prod(size)))
+    output = output.reshape((batch,) + tuple(size) + (-1,))
     return output, new_means, new_vars
 
 
@@ -121,7 +121,7 @@ class BatchNorm(hk.Module):
     """Batch normalization for orthonormal representations
     It normalizes by the norm of the representations.
     Note that the norm is invariant only for orthonormal representations.
-    Irreducible representations `wigner_D` are orthonormal.
+    Irreducible representations `e3nn_jax.wigner_D` are orthonormal.
 
     Args:
         irreps: Irreducible representations of the input and output (unchanged)
@@ -144,7 +144,7 @@ class BatchNorm(hk.Module):
         affine: bool = True,
         reduce: str = "mean",
         instance: bool = False,
-        normalization: str = "component",
+        normalization: str = None,
     ):
         super().__init__()
 
@@ -160,6 +160,8 @@ class BatchNorm(hk.Module):
         assert reduce in ["mean", "max"], "reduce needs to be 'mean' or 'max'"
         self.reduce = reduce
 
+        if normalization is None:
+            normalization = config("irrep_normalization")
         assert normalization in ["norm", "component"], "normalization needs to be 'norm' or 'component'"
         self.normalization = normalization
 
@@ -177,9 +179,9 @@ class BatchNorm(hk.Module):
             output: normalized tensor of shape ``(batch, [spatial], irreps.dim)``
         """
         if self.irreps is not None:
-            input = IrrepsData.new(self.irreps, input)
-        if not isinstance(input, IrrepsData):
-            raise ValueError("input should be of type IrrepsData")
+            input = IrrepsArray.from_any(self.irreps, input)
+        if not isinstance(input, IrrepsArray):
+            raise ValueError("input should be of type IrrepsArray")
 
         num_scalar = sum(mul for mul, ir in input.irreps if ir.is_scalar())
         num_features = input.irreps.num_irreps
