@@ -1,20 +1,240 @@
 """Defines the functional tensor product."""
 
+import collections
 import itertools
 from functools import lru_cache, partial
 from math import sqrt
-import collections
-from typing import Any, List, Dict, Optional
+from typing import Any, List, Optional, Union
 
 import jax
 import jax.numpy as jnp
 
-from e3nn_jax import Irreps, IrrepsData, clebsch_gordan, Instruction
+from e3nn_jax import Instruction, Irreps, IrrepsArray, clebsch_gordan, config
 from e3nn_jax.util import prod
 
 from ._einsum import einsum as opt_einsum
 
 
+class FunctionalTensorProduct:
+    r"""Tensor product of two tensors.
+
+    Args:
+        irreps_in1: :class:`~e3nn_jax.Irreps` of the first tensor.
+        irreps_in2: :class:`~e3nn_jax.Irreps` of the second tensor.
+        irreps_out: :class:`~e3nn_jax.Irreps` of the output tensor.
+        instructions: List of instructions.
+            ``[(i_in1, i_in2, i_out, connection_mode, has_weight, (path_weight)), ...]``
+            - i_in1, i_in2, i_out are indices of the irreps_in1, irreps_in2, irreps_out.
+            - connection_mode is one of ``uvw``, ``uvu``, ``uvv``, ``uuw``, ``uuu``, ``uvuv``
+            - has_weight is a boolean indicating whether the instruction has a weight.
+            - path_weight (optional, 1.0 by default) is the weight of the path.
+
+        in1_var: Variance of the first tensor.
+        in2_var: Variance of the second tensor.
+        out_var: Variance of the output tensor.
+        irrep_normalization: Normalization of the tensors. ``component`` or ``norm``.
+        path_normalization (str or float): Normalization of the paths, ``element`` or ``path``.
+            0/1 corresponds to a normalization where each element/path has an equal contribution to the forward propagation.
+        gradient_normalization (str or float): Normalization of the gradients, ``element`` or ``path``.
+            0/1 corresponds to a normalization where each element/path has an equal contribution to the learning.
+    """
+    irreps_in1: Irreps
+    irreps_in2: Irreps
+    irreps_out: Irreps
+    instructions: List[Instruction]
+    output_mask: jnp.ndarray
+
+    def __init__(
+        self,
+        irreps_in1: Any,
+        irreps_in2: Any,
+        irreps_out: Any,
+        instructions: List[Any],
+        in1_var: Optional[List[float]] = None,
+        in2_var: Optional[List[float]] = None,
+        out_var: Optional[List[float]] = None,
+        irrep_normalization: str = None,
+        path_normalization: Union[str, float] = None,
+        gradient_normalization: Union[str, float] = None,
+    ):
+        self.irreps_in1 = Irreps(irreps_in1)
+        self.irreps_in2 = Irreps(irreps_in2)
+        self.irreps_out = Irreps(irreps_out)
+        del irreps_in1, irreps_in2, irreps_out
+
+        instructions = [x if len(x) == 6 else x + (1.0,) for x in instructions]
+        instructions = [
+            Instruction(
+                i_in1,
+                i_in2,
+                i_out,
+                connection_mode,
+                has_weight,
+                path_weight,
+                None,
+                self.irreps_in1[i_in1].mul,
+                self.irreps_in2[i_in2].mul,
+                self.irreps_out[i_out].mul,
+            )
+            for i_in1, i_in2, i_out, connection_mode, has_weight, path_weight in instructions
+        ]
+
+        if in1_var is None:
+            in1_var = [1.0 for _ in self.irreps_in1]
+
+        if in2_var is None:
+            in2_var = [1.0 for _ in self.irreps_in2]
+
+        if out_var is None:
+            out_var = [1.0 for _ in self.irreps_out]
+
+        if irrep_normalization is None:
+            irrep_normalization = config("irrep_normalization")
+
+        if path_normalization is None:
+            path_normalization = config("path_normalization")
+        if isinstance(path_normalization, str):
+            path_normalization = {"element": 0.0, "path": 1.0}[path_normalization]
+
+        if gradient_normalization is None:
+            gradient_normalization = config("gradient_normalization")
+        if isinstance(gradient_normalization, str):
+            gradient_normalization = {"element": 0.0, "path": 1.0}[gradient_normalization]
+
+        self.instructions = _normalize_instruction_path_weights(
+            instructions,
+            self.irreps_in1,
+            self.irreps_in2,
+            self.irreps_out,
+            in1_var,
+            in2_var,
+            out_var,
+            irrep_normalization,
+            path_normalization,
+            gradient_normalization,
+        )
+
+        with jax.ensure_compile_time_eval():
+            if self.irreps_out.dim > 0:
+                self.output_mask = jnp.concatenate(
+                    [
+                        jnp.ones(mul_ir.dim)
+                        if any(
+                            (ins.i_out == i_out) and (ins.path_weight != 0) and (0 not in ins.path_shape)
+                            for ins in self.instructions
+                        )
+                        else jnp.zeros(mul_ir.dim)
+                        for i_out, mul_ir in enumerate(self.irreps_out)
+                    ]
+                )
+            else:
+                self.output_mask = jnp.ones(0)
+
+    def left_right(
+        self,
+        weights: List[jnp.ndarray],
+        input1: IrrepsArray,
+        input2: IrrepsArray = None,
+        *,
+        specialized_code=None,
+        optimize_einsums=None,
+        custom_einsum_vjp=None,
+        fuse_all=None,
+    ) -> IrrepsArray:
+        r"""Compute the tensor product of two input tensors.
+
+        Args:
+            weights (array or list of arrays): The weights of the tensor product.
+            input1 (IrrepsArray): The first input tensor.
+            input2 (IrrepsArray): The second input tensor.
+            specialized_code (bool): If True, use the specialized code for the
+                tensor product.
+            optimize_einsums (bool): If True, optimize the einsum code.
+            custom_einsum_vjp (bool): If True, use the custom vjp for the einsum
+                code.
+            fuse_all (bool): If True, fuse all the einsums.
+
+        Returns:
+            `IrrepsArray`: The output tensor.
+        """
+        if specialized_code is None:
+            specialized_code = config("specialized_code")
+        if optimize_einsums is None:
+            optimize_einsums = config("optimize_einsums")
+        if custom_einsum_vjp is None:
+            custom_einsum_vjp = config("custom_einsum_vjp")
+        if fuse_all is None:
+            fuse_all = config("fuse_all")
+
+        if input2 is None:
+            weights, input1, input2 = [], weights, input1
+
+        input1 = IrrepsArray.from_any(self.irreps_in1, input1)
+        input2 = IrrepsArray.from_any(self.irreps_in2, input2)
+
+        return _left_right(
+            self,
+            weights,
+            input1,
+            input2,
+            specialized_code=specialized_code,
+            optimize_einsums=optimize_einsums,
+            custom_einsum_vjp=custom_einsum_vjp,
+            fuse_all=fuse_all,
+        )
+
+    def right(
+        self,
+        weights: List[jnp.ndarray],
+        input2: IrrepsArray = None,
+        *,
+        specialized_code=None,
+        optimize_einsums=None,
+        custom_einsum_vjp=None,
+        fuse_all=None,
+    ) -> jnp.ndarray:
+        r"""Compute the right contraction of the tensor product.
+
+        Args:
+            weights (array or list of arrays): The weights of the tensor product.
+            input2 (IrrepsArray): The second input tensor.
+            optimize_einsums (bool): If True, optimize the einsum code.
+            custom_einsum_vjp (bool): If True, use the custom vjp for the einsum code.
+
+        Returns:
+            A matrix of shape ``(irreps_in1.dim, irreps_out.dim)``.
+        """
+        if specialized_code is None:
+            specialized_code = config("specialized_code")
+        if optimize_einsums is None:
+            optimize_einsums = config("optimize_einsums")
+        if custom_einsum_vjp is None:
+            custom_einsum_vjp = config("custom_einsum_vjp")
+        if fuse_all is None:
+            fuse_all = config("fuse_all")
+
+        if input2 is None:
+            weights, input2 = [], weights
+
+        input2 = IrrepsArray.from_any(self.irreps_in2, input2)
+        return _right(
+            self,
+            weights,
+            input2,
+            optimize_einsums=optimize_einsums,
+            custom_einsum_vjp=custom_einsum_vjp,
+        )
+
+    def __repr__(self):
+        npath = sum(prod(i.path_shape) for i in self.instructions)
+        nweight = sum(prod(i.path_shape) for i in self.instructions if i.has_weight)
+        return (
+            f"{self.__class__.__name__}"
+            f"({self.irreps_in1.simplify()} x {self.irreps_in2.simplify()} "
+            f"-> {self.irreps_out.simplify()} | {npath} paths | {nweight} weights)"
+        )
+
+
 def _sum_tensors(xs, shape, empty_return_none=False):
     xs = [x for x in xs if x is not None]
     if len(xs) > 0:
@@ -35,296 +255,69 @@ def _flat_concatenate(xs):
     return jnp.zeros((0,))
 
 
-def _compute_element_path_normalization_factors(
+def _normalize_instruction_path_weights(
     instructions: List[Instruction],
+    first_input_irreps: Irreps,
+    second_input_irreps: Irreps,
+    output_irreps: Irreps,
     first_input_variance: List[float],
     second_input_variance: List[float],
-) -> Dict[Instruction, float]:
-    """Returns a dictionary with keys as the Instructions and values as the corresponding path normalization factor
-    for 'element' path normalization."""
+    output_variance: List[float],
+    irrep_normalization: str,
+    path_normalization_exponent: float,
+    gradient_normalization_exponent: float,
+) -> List[Instruction]:
+    """Returns instructions with normalized path weights."""
+
+    def var(instruction):
+        return first_input_variance[instruction.i_in1] * second_input_variance[instruction.i_in2] * instruction.num_elements
+
+    # Precompute normalization factors.
     path_normalization_sums = collections.defaultdict(lambda: 0.0)
     for instruction in instructions:
-        path_normalization_sums[instruction.i_out] += (
-            first_input_variance[instruction.i_in1] * second_input_variance[instruction.i_in2] * instruction.num_elements
-        )
+        path_normalization_sums[instruction.i_out] += var(instruction) ** (1.0 - path_normalization_exponent)
 
-    return {instruction: path_normalization_sums[instruction.i_out] for instruction in instructions}
-
-
-def _compute_standard_path_normalization_factors(
-    instructions: List[Instruction],
-    first_input_variance: List[float],
-    second_input_variance: List[float],
-) -> Dict[Instruction, float]:
-    """Returns a dictionary with keys as the Instructions and values as the corresponding path normalization factor
-    for 'path' path normalization."""
-    path_normalization_counts = collections.defaultdict(lambda: 0.0)
-    for instruction in instructions:
-        path_normalization_counts[instruction.i_out] += 1
-
-    return {
-        instruction: first_input_variance[instruction.i_in1]
-        * second_input_variance[instruction.i_in2]
-        * instruction.num_elements
-        * path_normalization_counts[instruction.i_out]
+    path_normalization_factors = {
+        instruction: var(instruction) ** path_normalization_exponent * path_normalization_sums[instruction.i_out]
         for instruction in instructions
     }
 
+    def update(instruction: Instruction) -> float:
+        """Computes normalized path weight for a single instructions, with precomputed path normalization factors."""
 
-def normalize_instruction_path_weights(
-    instructions: List[Instruction],
-    first_input_irreps: Irreps,
-    second_input_irreps: Irreps,
-    output_irreps: Irreps,
-    first_input_variance: List[float],
-    second_input_variance: List[float],
-    output_variance: List[float],
-    irrep_normalization: str,
-    path_normalization: str,
-) -> List[Instruction]:
-    """Returns instructions with normalized path weights."""
-    # Precompute normalization factors.
-    if path_normalization == "element":
-        path_normalization_factors = _compute_element_path_normalization_factors(
-            instructions, first_input_variance, second_input_variance
-        )
-    elif path_normalization == "path":
-        path_normalization_factors = _compute_standard_path_normalization_factors(
-            instructions, first_input_variance, second_input_variance
-        )
-    else:
-        raise ValueError(f"Unsupported path normalization: {path_normalization}.")
+        if irrep_normalization not in ["component", "norm", "none"]:
+            raise ValueError(f"Unsupported irrep normalization: {irrep_normalization}.")
 
-    def compute_normalized_path_weight_fn(instruction: Instruction) -> float:
-        return compute_normalized_path_weight(
-            instruction,
-            first_input_irreps,
-            second_input_irreps,
-            output_irreps,
-            output_variance,
-            irrep_normalization,
-            path_normalization_factors,
-        )
+        mul_ir_in1 = first_input_irreps[instruction.i_in1]
+        mul_ir_in2 = second_input_irreps[instruction.i_in2]
+        mul_ir_out = output_irreps[instruction.i_out]
 
-    return [instruction.replace(path_weight=compute_normalized_path_weight_fn(instruction)) for instruction in instructions]
+        assert mul_ir_in1.ir.p * mul_ir_in2.ir.p == mul_ir_out.ir.p
+        assert abs(mul_ir_in1.ir.l - mul_ir_in2.ir.l) <= mul_ir_out.ir.l <= mul_ir_in1.ir.l + mul_ir_in2.ir.l
 
+        if irrep_normalization == "component":
+            alpha = mul_ir_out.ir.dim
+        if irrep_normalization == "norm":
+            alpha = mul_ir_in1.ir.dim * mul_ir_in2.ir.dim
+        if irrep_normalization == "none":
+            alpha = 1
 
-def compute_normalized_path_weight(
-    instruction: Instruction,
-    first_input_irreps: Irreps,
-    second_input_irreps: Irreps,
-    output_irreps: Irreps,
-    output_variance: List[float],
-    irrep_normalization: str,
-    path_normalization_factors: Dict[Instruction, float],
-) -> float:
-    """Computes normalized path weight for a single instructions, with precomputed path normalization factors."""
+        x = path_normalization_factors[instruction]
+        if x > 0.0:
+            alpha /= x
 
-    if irrep_normalization not in ["component", "norm", "none"]:
-        raise ValueError(f"Unsupported irrep normalization: {irrep_normalization}.")
+        alpha *= output_variance[instruction.i_out]
+        alpha *= instruction.path_weight
 
-    mul_ir_in1 = first_input_irreps[instruction.i_in1]
-    mul_ir_in2 = second_input_irreps[instruction.i_in2]
-    mul_ir_out = output_irreps[instruction.i_out]
-
-    assert mul_ir_in1.ir.p * mul_ir_in2.ir.p == mul_ir_out.ir.p
-    assert abs(mul_ir_in1.ir.l - mul_ir_in2.ir.l) <= mul_ir_out.ir.l <= mul_ir_in1.ir.l + mul_ir_in2.ir.l
-
-    if irrep_normalization == "component":
-        alpha = mul_ir_out.ir.dim
-    if irrep_normalization == "norm":
-        alpha = mul_ir_in1.ir.dim * mul_ir_in2.ir.dim
-    if irrep_normalization == "none":
-        alpha = 1
-
-    path_normalization_factor = path_normalization_factors[instruction]
-    if path_normalization_factor > 0.0:
-        alpha /= path_normalization_factor
-
-    alpha *= output_variance[instruction.i_out]
-    alpha *= instruction.path_weight
-    return sqrt(alpha)
-
-
-class FunctionalTensorProduct:
-    r"""Tensor product of two tensors.
-
-    Args:
-        irreps_in1: :class:`~e3nn_jax.Irreps` of the first tensor.
-        irreps_in2: :class:`~e3nn_jax.Irreps` of the second tensor.
-        irreps_out: :class:`~e3nn_jax.Irreps` of the output tensor.
-        instructions: List of instructions.
-            ``[(i_in1, i_in2, i_out, connection_mode, has_weight, (path_weight)), ...]``
-            - i_in1, i_in2, i_out are indices of the irreps_in1, irreps_in2, irreps_out.
-            - connection_mode is one of ``uvw``, ``uvu``, ``uvv``, ``uuw``, ``uuu``, ``uvuv``
-            - has_weight is a boolean indicating whether the instruction has a weight.
-            - path_weight (optional) is the weight of the path.
-
-        in1_var: Variance of the first tensor.
-        in2_var: Variance of the second tensor.
-        out_var: Variance of the output tensor.
-        irrep_normalization: Normalization of the tensors. `component` or `norm`.
-        path_normalization: Normalization of the paths. `element` or `path`.
-    """
-    irreps_in1: Irreps
-    irreps_in2: Irreps
-    irreps_out: Irreps
-    instructions: List[Instruction]
-    output_mask: jnp.ndarray
-
-    def __init__(
-        self,
-        irreps_in1: Any,
-        irreps_in2: Any,
-        irreps_out: Any,
-        instructions: List[Any],
-        in1_var: Optional[List[float]] = None,
-        in2_var: Optional[List[float]] = None,
-        out_var: Optional[List[float]] = None,
-        irrep_normalization: str = "component",
-        path_normalization: str = "element",
-    ):
-        self.irreps_in1 = Irreps(irreps_in1)
-        self.irreps_in2 = Irreps(irreps_in2)
-        self.irreps_out = Irreps(irreps_out)
-        del irreps_in1, irreps_in2, irreps_out
-
-        instructions = [x if len(x) == 6 else x + (1.0,) for x in instructions]
-        instructions = [
-            Instruction(
-                i_in1,
-                i_in2,
-                i_out,
-                connection_mode,
-                has_weight,
-                path_weight,
-                self.irreps_in1[i_in1].mul,
-                self.irreps_in2[i_in2].mul,
-                self.irreps_out[i_out].mul,
+        if instruction.has_weight:
+            return instruction.replace(
+                path_weight=sqrt(alpha) ** gradient_normalization_exponent,
+                weight_std=sqrt(alpha) ** (1.0 - gradient_normalization_exponent),
             )
-            for i_in1, i_in2, i_out, connection_mode, has_weight, path_weight in instructions
-        ]
+        else:
+            return instruction.replace(path_weight=sqrt(alpha))
 
-        if in1_var is None:
-            in1_var = [1.0 for _ in self.irreps_in1]
-
-        if in2_var is None:
-            in2_var = [1.0 for _ in self.irreps_in2]
-
-        if out_var is None:
-            out_var = [1.0 for _ in self.irreps_out]
-
-        self.instructions = normalize_instruction_path_weights(
-            instructions,
-            self.irreps_in1,
-            self.irreps_in2,
-            self.irreps_out,
-            in1_var,
-            in2_var,
-            out_var,
-            irrep_normalization,
-            path_normalization,
-        )
-
-        with jax.ensure_compile_time_eval():
-            if self.irreps_out.dim > 0:
-                self.output_mask = jnp.concatenate(
-                    [
-                        jnp.ones(mul_ir.dim)
-                        if any(
-                            (ins.i_out == i_out) and (ins.path_weight != 0) and (0 not in ins.path_shape)
-                            for ins in self.instructions
-                        )
-                        else jnp.zeros(mul_ir.dim)
-                        for i_out, mul_ir in enumerate(self.irreps_out)
-                    ]
-                )
-            else:
-                self.output_mask = jnp.ones(0)
-
-    def left_right(
-        self,
-        weights: List[jnp.ndarray],
-        input1: IrrepsData,
-        input2: IrrepsData = None,
-        *,
-        specialized_code=False,
-        optimize_einsums=True,
-        custom_einsum_vjp=False,
-        fuse_all=False,
-    ) -> IrrepsData:
-        r"""Compute the tensor product of two input tensors.
-
-        Args:
-            weights (array or list of arrays): The weights of the tensor product.
-            input1 (IrrepsData): The first input tensor.
-            input2 (IrrepsData): The second input tensor.
-            specialized_code (bool): If True, use the specialized code for the
-                tensor product.
-            optimize_einsums (bool): If True, optimize the einsum code.
-            custom_einsum_vjp (bool): If True, use the custom vjp for the einsum
-                code.
-            fuse_all (bool): If True, fuse all the einsums.
-
-        Returns:
-            `IrrepsData`: The output tensor.
-        """
-        if input2 is None:
-            weights, input1, input2 = [], weights, input1
-
-        input1 = IrrepsData.new(self.irreps_in1, input1)
-        input2 = IrrepsData.new(self.irreps_in2, input2)
-
-        return _left_right(
-            self,
-            weights,
-            input1,
-            input2,
-            specialized_code=specialized_code,
-            optimize_einsums=optimize_einsums,
-            custom_einsum_vjp=custom_einsum_vjp,
-            fuse_all=fuse_all,
-        )
-
-    def right(
-        self,
-        weights: List[jnp.ndarray],
-        input2: IrrepsData = None,
-        *,
-        optimize_einsums=False,
-        custom_einsum_vjp=False,
-    ) -> jnp.ndarray:
-        r"""Compute the right contraction of the tensor product.
-
-        Args:
-            weights (array or list of arrays): The weights of the tensor product.
-            input2 (IrrepsData): The second input tensor.
-            optimize_einsums (bool): If True, optimize the einsum code.
-            custom_einsum_vjp (bool): If True, use the custom vjp for the einsum code.
-
-        Returns:
-            A matrix of shape ``(irreps_in1.dim, irreps_out.dim)``.
-        """
-        if input2 is None:
-            weights, input2 = [], weights
-
-        input2 = IrrepsData.new(self.irreps_in2, input2)
-        return _right(
-            self,
-            weights,
-            input2,
-            optimize_einsums=optimize_einsums,
-            custom_einsum_vjp=custom_einsum_vjp,
-        )
-
-    def __repr__(self):
-        npath = sum(prod(i.path_shape) for i in self.instructions)
-        nweight = sum(prod(i.path_shape) for i in self.instructions if i.has_weight)
-        return (
-            f"{self.__class__.__name__}"
-            f"({self.irreps_in1.simplify()} x {self.irreps_in2.simplify()} "
-            f"-> {self.irreps_out.simplify()} | {npath} paths | {nweight} weights)"
-        )
+    return [update(instruction) for instruction in instructions]
 
 
 @partial(
@@ -345,7 +338,7 @@ def _left_right(
 
     # = Short-circut for zero dimensional =
     if self.irreps_in1.dim == 0 or self.irreps_in2.dim == 0 or self.irreps_out.dim == 0:
-        return IrrepsData.zeros(self.irreps_out, ())
+        return IrrepsArray.zeros(self.irreps_out, ())
 
     if custom_einsum_vjp:
         assert optimize_einsums
@@ -372,8 +365,8 @@ def _left_right(
         assert i == weights.size
     del weights
 
-    assert len(input1.shape) == 0
-    assert len(input2.shape) == 0
+    assert input1.ndim == 1, f"input1 is shape {input1.shape}. Execting ndim to be 1. Use jax.vmap to map over input1"
+    assert input2.ndim == 1, f"input2 is shape {input2.shape}. Execting ndim to be 1. Use jax.vmap to map over input2"
 
     if fuse_all:
         with jax.ensure_compile_time_eval():
@@ -440,7 +433,7 @@ def _left_right(
 
         if has_path_with_no_weights and big_w3j.shape[0] == 1:
             big_w3j = big_w3j.reshape(big_w3j.shape[1:])
-            out = einsum("ijk,i,j->k", big_w3j, input1.contiguous, input2.contiguous)
+            out = einsum("ijk,i,j->k", big_w3j, input1.array, input2.array)
         else:
             if has_path_with_no_weights:
                 weights_flat = jnp.concatenate([jnp.ones((1,)), weights_flat])
@@ -449,10 +442,10 @@ def _left_right(
                 "p,pijk,i,j->k",
                 weights_flat,
                 big_w3j,
-                input1.contiguous,
-                input2.contiguous,
+                input1.array,
+                input2.array,
             )
-        return IrrepsData.from_contiguous(self.irreps_out, out)
+        return IrrepsArray(self.irreps_out, out)
 
     @lru_cache(maxsize=None)
     def multiply(in1, in2, mode):
@@ -605,7 +598,7 @@ def _left_right(
         )
         for i_out, mul_ir_out in enumerate(self.irreps_out)
     ]
-    return IrrepsData.from_list(self.irreps_out, out, ())
+    return IrrepsArray.from_list(self.irreps_out, out, ())
 
 
 @partial(jax.jit, static_argnums=(0,), static_argnames=("optimize_einsums", "custom_einsum_vjp"))
