@@ -57,6 +57,10 @@ class Irrep:
                 p = l.p
                 l = l.l
 
+            if isinstance(l, MulIrrep):
+                p = l.ir.p
+                l = l.ir.l
+
             if isinstance(l, str):
                 try:
                     name = l.strip()
@@ -193,12 +197,19 @@ class Irrep:
     def __add__(self, other):
         return Irreps(self) + Irreps(other)
 
+    def __radd__(self, other):
+        return Irreps(other) + Irreps(self)
+
     def __iter__(self):
         yield self.l
         yield self.p
 
     def __lt__(self, other):
-        return (self.l, self.p) < (other.l, other.p)
+        return (self.l, -self.p * (-1) ** self.l) < (other.l, -other.p * (-1) ** other.l)
+
+    def __eq__(self, other: object) -> bool:
+        other = Irrep(other)
+        return (self.l, self.p) == (other.l, other.p)
 
 
 jax.tree_util.register_pytree_node(Irrep, lambda ir: ((), ir), lambda ir, _: ir)
@@ -425,6 +436,9 @@ class Irreps(tuple):
         irreps = Irreps(irreps)
         return Irreps(super().__add__(irreps))
 
+    def __radd__(self, irreps):
+        return Irreps(irreps) + self
+
     def __mul__(self, other):
         r"""
         >>> (Irreps('2x1e') * 3).simplify()
@@ -441,6 +455,17 @@ class Irreps(tuple):
         """
         return Irreps(super().__rmul__(other))
 
+    def __eq__(self, other: object) -> bool:
+        if isinstance(other, str):
+            try:
+                other = Irreps(other)
+            except ValueError:
+                return False
+        return super().__eq__(other)
+
+    def __hash__(self) -> int:
+        return super().__hash__()
+
     def unify(self):
         r"""Regroup same irrep together.
 
@@ -543,6 +568,7 @@ class Irreps(tuple):
 
     @property
     def num_irreps(self) -> int:
+        """Sum of the multiplicities."""
         return sum(mul for mul, _ in self)
 
     @property
