@@ -1,5 +1,5 @@
 import math
-from typing import List, Optional, Tuple
+from typing import List, Optional, Tuple, Union
 
 import jax
 import jax.numpy as jnp
@@ -83,12 +83,18 @@ class IrrepsArray:
             IrrepsArray
         """
         irreps = Irreps(irreps)
-        assert len(irreps) == len(list), f"irreps has {len(irreps)} elements and list has {len(list)}"
-        assert all(x is None or isinstance(x, jnp.ndarray) for x in list)
-        assert all(x is None or x.shape == leading_shape + (mul, ir.dim) for x, (mul, ir) in zip(list, irreps)), (
-            [x.shape for x in list if x is not None],
-            leading_shape,
-        )
+        if len(irreps) != len(list):
+            raise ValueError(f"IrrepsArray.from_list: len(irreps) != len(list), {len(irreps)} != {len(list)}")
+
+        if not all(x is None or isinstance(x, jnp.ndarray) for x in list):
+            raise ValueError(f"IrrepsArray.from_list: list contains non-array elements type={[type(x) for x in list]}")
+
+        if not all(x is None or x.shape == leading_shape + (mul, ir.dim) for x, (mul, ir) in zip(list, irreps)):
+            raise ValueError(
+                f"IrrepsArray.from_list: list shapes {[None if x is None else x.shape for x in list]} "
+                f"incompatible with leading shape {leading_shape} and irreps {irreps}. "
+                f"Expecting {[leading_shape + (mul, ir.dim) for (mul, ir) in irreps]}."
+            )
 
         if irreps.dim > 0:
             array = jnp.concatenate(
@@ -140,12 +146,18 @@ class IrrepsArray:
         )
 
     def __repr__(self):
-        return f"{self.irreps}\n{self.array}"
+        r = str(self.array)
+        if "\n" in r:
+            return f"{self.irreps}\n{r}"
+        return f"{self.irreps} {r}"
 
     @property
     def shape(self):
         return self.array.shape
 
+    def __len__(self):
+        return len(self.array)
+
     @property
     def ndim(self):
         return len(self.shape)
@@ -181,13 +193,35 @@ class IrrepsArray:
     def simplify(self) -> "IrrepsArray":
         return self.convert(self.irreps.simplify())
 
-    def split(self, indices: List[int]) -> List["IrrepsArray"]:
-        array_parts = jnp.split(self.array, [self.irreps[:i].dim for i in indices], axis=-1)
-        assert len(array_parts) == len(indices) + 1
-        return [
-            IrrepsArray(irreps=self.irreps[i:j], array=array, list=self.list[i:j])
-            for (i, j), array in zip(zip([0] + indices, indices + [len(self.irreps)]), array_parts)
-        ]
+    def sorted(self) -> "IrrepsArray":
+        irreps, p, inv = self.irreps.sort()
+        return IrrepsArray.from_list(irreps, [self.list[i] for i in inv], self.shape[:-1])
+
+    def split(self, indices: Union[List[int], List[Irreps]]) -> List["IrrepsArray"]:
+        """Split the array into subarrays
+
+        Examples:
+            >>> IrrepsArray("0e + 1e", jnp.array([1.0, 2, 3, 4])).split(["0e", "1e"])
+            [1x0e [1.], 1x1e [2. 3. 4.]]
+
+            >>> IrrepsArray("0e + 1e", jnp.array([1.0, 2, 3, 4])).split([1])
+            [1x0e [1.], 1x1e [2. 3. 4.]]
+        """
+        if all(isinstance(i, int) for i in indices):
+            array_parts = jnp.split(self.array, [self.irreps[:i].dim for i in indices], axis=-1)
+            assert len(array_parts) == len(indices) + 1
+            return [
+                IrrepsArray(irreps=self.irreps[i:j], array=array, list=self.list[i:j])
+                for (i, j), array in zip(zip([0] + indices, indices + [len(self.irreps)]), array_parts)
+            ]
+
+        irrepss = [Irreps(i) for i in indices]
+        assert self.irreps.simplify() == sum(irrepss, Irreps()).simplify()
+        array_parts = jnp.split(
+            self.array, [sum(irreps.dim for irreps in irrepss[:i]) for i in range(1, len(irrepss))], axis=-1
+        )
+        assert len(array_parts) == len(irrepss)
+        return [IrrepsArray(irreps, array) for irreps, array in zip(irrepss, array_parts)]
 
     def __getitem__(self, index) -> "IrrepsArray":
         if not isinstance(index, tuple):
@@ -201,13 +235,42 @@ class IrrepsArray:
             list=[None if x is None else x[index] for x in self.list],
         )
 
+    def __eq__(self, other: object) -> "IrrepsArray":
+        assert self.irreps == other.irreps
+
+        leading_shape = jnp.broadcast_shapes(self.shape[:-1], other.shape[:-1])
+
+        def eq(mul: int, x: jnp.ndarray, y: jnp.ndarray) -> jnp.ndarray:
+            if x is None or y is None:
+                return jnp.ones(leading_shape + (mul,), dtype="bool")
+            if x is not None and y is not None:
+                return jnp.all(x == y, axis=-1)
+            return jnp.zeros(leading_shape + (mul,), dtype="bool")
+
+        list = [eq(mul, x, y)[..., None] for (mul, ir), x, y in zip(self.irreps, self.list, other.list)]
+        return IrrepsArray.from_list([(mul, "0e") for mul, _ in self.irreps], list, leading_shape)
+
     def repeat_irreps_by_last_axis(self) -> "IrrepsArray":
+        r"""Repeat the irreps by the last axis of the array.
+
+        Example:
+            >>> irreps = IrrepsArray("0e + 1e", jnp.zeros((3, 4)))
+            >>> irreps.repeat_irreps_by_last_axis().irreps
+            1x0e+1x1e+1x0e+1x1e+1x0e+1x1e
+        """
         assert len(self.shape) >= 2
         irreps = (self.shape[-2] * self.irreps).simplify()
         array = self.array.reshape(self.shape[:-2] + (irreps.dim,))
         return IrrepsArray(irreps, array)
 
     def repeat_mul_by_last_axis(self) -> "IrrepsArray":
+        r"""Repeat the multiplicity by the last axis of the array.
+
+        Example:
+            >>> irreps = IrrepsArray("0e + 1e", jnp.zeros((3, 4)))
+            >>> irreps.repeat_mul_by_last_axis().irreps
+            3x0e+3x1e
+        """
         assert len(self.shape) >= 2
         irreps = Irreps([(self.shape[-2] * mul, ir) for mul, ir in self.irreps])
         list = [None if x is None else x.reshape(self.shape[:-2] + (mul, ir.dim)) for (mul, ir), x in zip(irreps, self.list)]
@@ -217,6 +280,13 @@ class IrrepsArray:
         raise NotImplementedError
 
     def factor_mul_to_last_axis(self, factor=None) -> "IrrepsArray":
+        r"""Create a new axis in the previous last position by factoring the multiplicities.
+
+        Example:
+            >>> irreps = IrrepsArray("6x0e + 3x1e", jnp.zeros((3, 6 + 9)))
+            >>> irreps.factor_mul_to_last_axis().irreps
+            2x0e+1x1e
+        """
         if factor is None:
             factor = math.gcd(*(mul for mul, _ in self.irreps))
 
